{% extends ":application/template.html" %}

{% block title %}Django-app-namespace-template-loader{% endblock title %}

{% block header %}Django-app-namespace-loader{% endblock header %}

{% block content %}This page has been generated by a combination of extend and override of these templates:{% endblock content %}

{% block list %}
<li>
  <code>application_extension:application/template.html</code>
</li>
{{ block.super }}
{% endblock%}
